# -*- coding: utf-8 -*-
from enigma import *
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.InputBox import InputBox
from Screens.ChoiceBox import ChoiceBox
from Components.ActionMap import ActionMap, NumberActionMap
from Components.ScrollLabel import ScrollLabel
from Components.GUIComponent import *
from Components.MenuList import MenuList
from Components.Input import Input
from Screens.Console import Console
from Plugins.Plugin import PluginDescriptor
from Components.Label import Label
from Components.Pixmap import Pixmap
from Tools.LoadPixmap import LoadPixmap
import Screens.Standby     
from enigma import quitMainloop, eConsoleAppContainer

import os
import gettext

freeze_version = "0.7.1"
freeze_plugindir="/usr/lib/enigma2/python/Plugins/Extensions/Freeze" 
global freeze_reboot_pending
freeze_reboot_pending=False

os.environ['LANGUAGE']='en'
freeze_language='en'

if os.path.exists("/etc/enigma2/settings") == True:
   f = open("/etc/enigma2/settings")
   line = f.readline()
   while (line):
	line = f.readline().replace("\n","")
	sp = line.split("=")
	if (sp[0] == "config.osd.language"):
	   sp2 = sp[1].split("_")
           freeze_language = sp2[0]
           if os.path.exists("%s/locale/%s" % (freeze_plugindir, freeze_language)) == True:
              os.environ["LANGUAGE"] = sp[1]
           else:
              os.environ['LANGUAGE']='en'
   f.close

_=gettext.Catalog('freeze', '%s/locale' % freeze_plugindir).gettext

def startFreeze(session, **kwargs):
    session.open(FreezePlugin)
        
def mainconf(menuid):
    if menuid != "setup":
    	return [ ]
    return [(_("Freeze & Extension"), startFreeze, "freeze", None)]

def autostart(reason,**kwargs):
    print "[FREEZE] autostart umountfs check"
    if os.path.exists("/etc/init.d/umountfs"):
    	f=open("/etc/init.d/umountfs","r")
    	u=f.read()
    	f.close()
    	if u.find("sync") is -1:
    		u=u+"\nsync\n"
	    	f=open("/etc/init.d/umountfs","w")
	    	f.write(u)
	    	f.close()

def Plugins(**kwargs):
    return [PluginDescriptor(name="Freeze Plugin", description=_("for extending and freezing your image"), where = PluginDescriptor.WHERE_MENU, fnc=mainconf),
	    PluginDescriptor(where = [PluginDescriptor.WHERE_SESSIONSTART, PluginDescriptor.WHERE_AUTOSTART], fnc = autostart)]

class FreezePlugin(Screen):
	skin = """
		<screen position="center,80" size="570,60" title="Freeze Plugin" >
		<widget name="buttonred" position="10,10" size="130,40" backgroundColor="red" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
		<widget name="buttongreen" position="150,10" size="130,40" backgroundColor="green" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
		<widget name="buttonyellow" position="290,10" size="130,40" backgroundColor="yellow" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
		<widget name="buttonblue" position="430,10" size="130,40" backgroundColor="blue" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
	</screen>"""
	def __init__(self, session, args = 0):
		Screen.__init__(self, session)
		self.onShown.append(self.setWindowTitle)      
		self.onLayoutFinish.append(self.byLayoutEnd)
		self["logo"] = Pixmap()
		self["buttonred"] = Label(_("Cancel"))
        	if os.path.exists("/.freeze"):
			self["buttonyellow"] = Label(_("Unfreeze"))
		else:
			self["buttonyellow"] = Label(_("Freeze"))
		self["buttongreen"] = Label(_("Info"))
		self["buttonblue"] = Label(_("About"))
		self["setupActions"] = ActionMap([ "ColorActions", "SetupActions" ],
			{
			"green": self.info,
			"red": self.leaving,
			"blue": self.about,
			"yellow": self.freezing,
			"save": self.leaving,
			"cancel": self.leaving,
			})

	def setWindowTitle(self):
	    	if os.path.exists("/.freeze"):
    			frozen=_("frozen")
 	   	else:	
    			frozen=_("not frozen")
		global freeze_reboot_pending
		if not freeze_reboot_pending:
	        	self.setTitle(_("Freeze Plugin in %s Flash") % (frozen))                  
	        
	def byLayoutEnd(self):                                                                     
		pass
	        
	def leaving(self):
		self.close()
        
	def about(self):
       		title=_("Freeze Plugin Version %s by gutemine") % (freeze_version)
       		self.session.open(MessageBox, title,  MessageBox.TYPE_INFO)

	def info(self):
		self.session.open(FreezeInfo)
		
	def freezing(self):
		global freeze_reboot_pending
		if os.path.exists("/.bainfo"):	      
            		self.skipFreeze(_("not supported in Barry Allen Image"))
		elif freeze_reboot_pending:	      
            		self.skipFreeze(_("reboot pending"))
            	else:
		    	if os.path.exists("/.freeze"):
		    		os.remove("/.freeze")
		    		if os.path.exists("/dev/ubi0_0"):
		    			os.system("mount -t ubifs /dev/ubi0_0 /boot/mnt/flash")
		    		else:
		    			os.system("mount -t jffs2 /dev/mtdblock3 /boot/mnt/flash")
			    	if os.path.exists("/boot/mnt/flash/.freeze"):
		    			os.remove("/boot/mnt/flash/.freeze")
		    		os.system("umount /boot/mnt/flash")
		    		self["buttonyellow"].setText(_("Freeze"))    
				self.session.openWithCallback(self.doReboot, MessageBox, _("reboot OK for unfreezing Flash ?"), MessageBox.TYPE_INFO)
		    	else:
				self.session.openWithCallback(self.askForDevice,ChoiceBox,_("choose freeze device"),self.getDeviceList())

	def getDeviceList(self):                                               
		found=0
		sp=[]
		f=open("/proc/partitions","r")
	       	devlist= []
	 	line = f.readline()                                                    
	 	line = f.readline()                                                    
	       	while (line):                                             
       			line = f.readline()                                                 
			if line.find("sd") is not -1:
				sp=line.split()
				devname="/dev/%s" % sp[3]
				if len(devname) > 8:
					size=int(sp[2])
					devsizeunit="MB"
					devsize=size/1024
					found=found+1
					devlist.append(("%s %d %s" % (devname,devsize,devsizeunit), devname,size))
		f.close()
		if found == 0:
			devlist.append(("no device found, shutdown, add device and reboot" , "nodev", 0))
	      	return devlist
        
    	def askForDevice(self,device):
    		if device is None:
            		self.skipFreeze(_("no device choosen"))
    		elif device[1] == "nodev":
            		self.skipFreeze(_("no device found"))
        	else:
        		self.device= device[1]
            		if os.path.exists(self.device):
    	    			self.session.openWithCallback(self.askForFreeze,MessageBox,(_("are you sure to freeze Flash to %s ?") % self.device), MessageBox.TYPE_YESNO)
    	    		else:
            			self.skipFreeze(_("%s not found, check partitioning") % self.device)
        
	def askForFreeze(self,answer):
        	if answer is None:
            		self.skipFreeze(_("answer is None"))
        	elif answer is False:
            		self.skipFreeze(_("you were not confirming"))
        	else:
			os.system("umount /boot/mnt/freeze")
			os.system("mount -o rw,remount /boot")
			if not os.path.exists("/boot/mnt"):	      
				os.mkdir("/boot/mnt")
			if not os.path.exists("/boot/mnt/root"):	      
				os.mkdir("/boot/mnt/root")
			if not os.path.exists("/boot/mnt/flash"):	      
				os.mkdir("/boot/mnt/flash")
			if not os.path.exists("/boot/mnt/freeze"):	      
				os.mkdir("/boot/mnt/freeze")
			else:
				os.system("rm -rf /boot/mnt/freeze")
				if not os.path.exists("/boot/mnt/freeze"):	      
					os.mkdir("/boot/mnt/freeze")
			os.system("mount -o rw,remount /boot")
			boxtype="dm7020hd"                     
			f=open("/proc/stb/info/model") 
			boxtype=f.read()                
			f.close()                           
			boxtype=boxtype.replace("\n","").replace("\l","")
			print "[FREEZE] boxtype %s" % boxtype
			if boxtype == "dm800":
				os.system("mount %s -t ext3 /boot/mnt/freeze" % self.device)
			else:
				os.system("mount %s -t ext4 /boot/mnt/freeze" % self.device)
			f=open("/proc/mounts") 
			mounts=f.read()                
			f.close()                           
			if mounts.find("/boot/mnt/freeze") is -1:
				print "[FREEZE] mount check failed"
	            		self.skipFreeze(_("Freeze device not usable,\nmaybe FAT instead of ext4/ext3 ?"))
	            	else:
				print "[FREEZE] mount check passed"
	            		try:
	            			os.symlink("/etc/fstab", "/boot/mnt/freeze/.check")
	            		except:
	            			pass
            			if not os.path.exists("/boot/mnt/freeze/.check"):
					print "[FREEZE] symlink check for FAT or filesystem full failed"
					os.system("umount /boot/mnt/freeze")
		            		self.skipFreeze(_("Freeze device not usable,\nmaybe FAT instead of ext4/ext3 ?"))
		            	else:
					print "[FREEZE] symlink check passed"
	            			os.remove("/boot/mnt/freeze/.check")
	            			try:
						f=open("/.freeze","w") 
						f.write("")                
						f.close()                           
					except:
						pass
            				if not os.path.exists("/.freeze"):
						print "[FREEZE] writing /.freeze failed"
						os.system("umount /boot/mnt/freeze")
		            			self.skipFreeze(_("Flash not freezable,\nmaybe already full ?"))
		            		else:
						print "[FREEZE] writing /.freeze passed"
	            				if os.path.exists("/boot/mnt/freeze/freeze"):
	            					os.system("rm -rf /boot/mnt/freeze/freeze")
	            					if os.path.exists("/boot/mnt/freeze/freeze"):
								print "[FREEZE] erase failed"
								os.system("umount /boot/mnt/freeze")
								os.remove("/.freeze")
					            		self.skipFreeze(_("Freeze erase failed,\nmaybe %s needs filesystemcheck ?") % self.device)
					            		return
	            					else:
								print "[FREEZE] erase succeeded"
	            				os.mkdir("/boot/mnt/freeze/freeze")
	            				os.system("umount /boot/mnt/freeze")
						self["buttonyellow"].setText(_("Unfreeze"))    
        					self.session.openWithCallback(self.doReboot, MessageBox, _("reboot OK for freezing Flash to %s ?") % self.device, MessageBox.TYPE_INFO)
           
    	def doReboot(self,reason):
    		if reason:
    			quitMainloop(2)               
    		else:
    			global freeze_reboot_pending
    			freeze_reboot_pending=True
 	       		self.session.open(MessageBox,_("reboot was cancelled,\nplease reboot later for finishing the action"), MessageBox.TYPE_WARNING)
 	       		
    	def skipFreeze(self,reason):
        	self.session.open(MessageBox,_("Freeze was canceled, because %s") % reason, MessageBox.TYPE_ERROR)

class FreezeInfo(Screen):
    skin = """
	<screen position="center,80" size="570,300" title="Freeze Info" >
	<widget name="frozen" position="320,80" size="180,180" transparent="1" alphatest="on" />
	<widget name="buttonred" position="10,10" size="130,40" backgroundColor="red" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
	<widget name="buttongreen" position="150,10" size="130,40" backgroundColor="green" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
	<widget name="buttonyellow" position="290,10" size="130,40" backgroundColor="yellow" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
	<widget name="buttonblue" position="430,10" size="130,40" backgroundColor="blue" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;18"/>
        <widget name="freefreeze" position="30,60" size="250,230" valign="center" halign="center" zPosition="2"  foregroundColor="white" font="Regular;20"/>
        </screen>"""

    def __init__(self, session, args = 0):
	Screen.__init__(self, session)
        self.onShown.append(self.setWindowTitle)
	self.onLayoutFinish.append(self.byLayoutEnd)
	self["logo"] = Pixmap()
	self["frozen"] = Pixmap()
       	self["buttonred"] = Label(_("Cancel"))
       	self["buttongreen"] = Label(_("OK"))
	self["buttonblue"] = Label(_("About"))
	f=[]
	device=_("no freeze device")
    	if os.path.exists("/.freeze"):
       		self["buttonyellow"] = Label(_("Check"))
		if os.path.exists("/dev/loop5"):
			os.system("losetup /dev/loop5 > /var/out")
			lo=open("/var/out")
			device = lo.readline()
        		lo.close()
        		device=device.replace("/dev/loop5: 0 ","").replace("\l","").replace("\n","")
        		print "[FREEZE] device %s" % device
			os.system("df -m | grep /dev/loop5 | tail -n 1 > /var/out")
			lo=open("/var/out")
			free = lo.readline()
        		lo.close()
        		free=free.replace("/dev/loop5","").replace("/","")
	else:
       		self["buttonyellow"] = Label(_("---"))
		os.system("df -m > /var/out")
		df=open("/var/out")
		free = df.readline()
		found=False
		while (free) and not found:
    			free = df.readline()
			if free.endswith("/\n"):
				found=True
       		df.close()
	while free.find("  ") is not -1:
		free=free.replace("  "," ")
	f=free.split(" ")
       	
	if len(f) > 3:
		freefreeze=_("Freeze Device:\n%s\n\nRoot Filesystem:\nTotal: %s MB\nUsed: %s MB\nFree: %s MB") % (device,f[1],f[2],f[3])
	else:
		freefreeze=_("no freeze device")
       	self["freefreeze"] = Label(freefreeze)
       	
        self["setupActions"] = ActionMap(["SetupActions", "ColorActions"],
       	{
       		"green": self.cancel,
        	"red": self.cancel,
	       	"yellow": self.check,
        	"blue": self.about,
            	"save": self.cancel,
            	"cancel": self.cancel,
            	"ok": self.cancel,
       	})
    def setWindowTitle(self):
	title=_("Freeze Plugin Version %s by gutemine") % (freeze_version)
        self.setTitle(title)
        
    def byLayoutEnd(self):                                                                     
    	if os.path.exists("/.freeze"):
	    	self["frozen"].instance.setPixmapFromFile("%s/frozen.png" % freeze_plugindir)
    	
    def cancel(self):
        self.close(False)
        
    def check(self):
	global freeze_reboot_pending
    	if os.path.exists("/.freeze"):
	    	if not os.path.exists("/.fsck"):
	        	try:
				f=open("/.fsck","w") 
				f.write("")                
				f.close()                           
			except:
				pass
       		if not os.path.exists("/.fsck"):
			print "[FREEZE] reguesting /.fsck failed"
	        	self.session.open(MessageBox,_("Freeze device not checkable\ntry boot without and check with Storage Manager later ?"), MessageBox.TYPE_ERROR)
       		else:
			self.session.openWithCallback(self.doReboot, MessageBox, _("reboot OK for checking Freeze device ?"), MessageBox.TYPE_INFO)
           
    def doReboot(self,reason):
    	if reason:
    		quitMainloop(2)               
    	else:
    		global freeze_reboot_pending
    		freeze_reboot_pending=True
 		self.session.open(MessageBox,_("reboot was cancelled,\nplease reboot later for finishing the action"), MessageBox.TYPE_WARNING)
 	       		
    def about(self):
     	title=_("Freeze Plugin Version %s by gutemine") % (freeze_version)
     	self.session.open(MessageBox, title,  MessageBox.TYPE_INFO)


